import java.util.Hashtable;
import java.util.Properties;

import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSession;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.apache.log4j.xml.DOMConfigurator;

public class SendMessageJNDI {

  //  private final String        destinationTopicName = "topic/MES";
  private final String    destinationTopicName = "topic/testopic";
  private Logger          lLog                 = Logger.getLogger(SendMessageJNDI.class);

  private TopicSession    session;
  private TopicConnection lConnection;
  private MessageProducer publisher;

  public static void main(String[] args){
    DOMConfigurator.configureAndWatch("log4j.xml");
    SendMessageJNDI lMain1 = new SendMessageJNDI();
    try {

      lMain1.initConnection(lMain1.destinationTopicName);
      String SEQ = "A";
      lMain1.sendMessage(SEQ + " - Hello ");

    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      try {
        lMain1.closeConnection();
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  public void initConnection(String destinationTopicName) throws Exception{
    TopicConnectionFactory lConFact = new ServiceLokup<TopicConnectionFactory>().doLookup("/RemoteConnectionFactory");
    lConnection = lConFact.createTopicConnection();
    lConnection.setClientID("CLIENT-1");
    session = lConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

    Topic lTopic = new ServiceLokup<Topic>().doLookup(destinationTopicName);
    publisher = getPublisher(lTopic);
    lConnection.start();
  }

  public TopicPublisher getPublisher(Topic pTopic){
    try {
      return getSession().createPublisher(pTopic);
    } catch (JMSException e) {
      lLog.error("", e);
      return null;
    }
  }

  public void sendMessage(String message){
    try {
      ObjectMessage lMessage = getSession().createObjectMessage(message);
      long currentTimeMillis = System.currentTimeMillis();
      publisher.send(lMessage);
      lLog.info("Message sent! " + currentTimeMillis + " " + getSession().hashCode());

    } catch (Exception e) {
      lLog.error("", e);
    }

  }

  private void closeConnection()
  {
    try
    {
      if (lConnection != null)
      {
        getSession().close();
        lConnection.close();

      }
    } catch (JMSException jmse)
    {
      lLog.info("Could not close connection " + lConnection + " exception was ", jmse);
    }
  }

  public TopicSession getSession(){
    return session;
  }

  public static class ServiceLokup<T> {
    private Logger       lLog  = Logger.getLogger(ServiceLokup.class);
    private final String lIP   = "localhost";
    private final String lPort = "1099";

    public T doLookup(String pServiceName){
      final Hashtable<String, String> jndiProperties = new Hashtable<String, String>();

      jndiProperties.put("remote.connectionprovider.create.options.org.xnio.Options.SSL_ENABLED", "false");
      jndiProperties.put("remote.connections", "default");
      jndiProperties.put("remote.connection.default.host", "localhost");
      jndiProperties.put("remote.connection.default.port", "4447");
      jndiProperties.put("remote.connection.default.connect.options.org.xnio.Options.SASL_POLICY_NOANONYMOUS", "false");
      try {
        final Context context = new InitialContext(jndiProperties);
        context.lookup(pServiceName);
      } catch (NamingException e) {
        lLog.error("Error: ", e);
        System.exit(0);
      }
      return null;

    }

    @SuppressWarnings("unchecked")
    public T doLookupOld(String pServiceName){
      InitialContext context;
      Properties properties = new Properties();
      properties.put("java.naming.factory.initial", "org.jnp.interfaces.NamingContextFactory");
      properties.put("java.naming.factory.url.pkgs", "org.jboss.naming:org.jnp.interfaces");
      properties.put("java.naming.provider.url", "jnp://" + lIP + ":" + lPort);
      try {
        context = new InitialContext(properties);
        return (T)context.lookup(pServiceName);
      } catch (NamingException e) {
        lLog.error("Error: ", e);
        System.exit(0);
      }
      return null;
    }
  }
}
